//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//PIMS: Personal Invstment Management System
//Class : Alerts
//This class defines two methods:-
//	1. checkForAlerts: Returns an array of pending alerts.
//	2. setNewAlert: Registers a new alert in its database set by the user.
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

import java.lang.*;
import java.io.*;
import java.util.*;
import java.text.*;

public class Alerts{

	//attributes.
	private dataRepository DR;
	private int numAlerts;//Total number of alerts specified by the user.
	private String alertList[][];//list of alerts. Contains alert date and alert details.
	private boolean debug = false;//boolean variable used for debugging this class.

	public Alerts(dataRepository DR){
	   try{
		this.DR = DR;

		//Read the alert file and initialize the alert list.
		alertList = DR.readAlerts();

		//Count the number of alerts.
		while(alertList[numAlerts][0] != null){
			numAlerts++;
		}
	   }catch(Exception e){}
	}

	//This method compares two dates. returns true if the first date is less than equal to
	//the second date and false otherwise.
	private boolean lessThanEqual(String date1, String date2){

		int day1, day2;
		int month1, month2;
		int year1, year2;

		StringTokenizer ST1 = new StringTokenizer(date1, "/");
		StringTokenizer ST2 = new StringTokenizer(date2, "/");

		//extract the day month and year for both the dates.
		day1 = Integer.parseInt(ST1.nextToken());
		day2 = Integer.parseInt(ST2.nextToken());
		month1 = Integer.parseInt(ST1.nextToken());
		month2 = Integer.parseInt(ST2.nextToken());
		year1 = Integer.parseInt(ST1.nextToken());
		year2 = Integer.parseInt(ST2.nextToken());

		//compare the dates.
		if(year1 < year2)return(true);
		if(year1 > year2)return(false);
		if(month1 < month2)return(true);
		if(month1 > month2)return(false);
		if(day1 < day2)return(true);
		if(day1 > day2)return(false);
		return(true);//the dates are equal.
	}

	//debugging method for printing.
	public void printArray(String array[][], int n){

		for(int i=0;i<n;++i){
			System.out.println(array[i][0]+" "+array[i][1]);
		}
	}

	//returns the nuber of alerts in the database
	public int numberOfAlerts(){
		return(numAlerts);
	}
	
	//This method returns the list of all alerts
	public String[][] readAlerts(){
		return(alertList);
	}

	//This method returns a list of pending alerts.
	public String[][] checkForAlerts()throws Exception{

		int i,j=0;
		String returnList[][] = new String[numAlerts + 1][2];
		long currentTime=(new Date()).getTime();
		long alertTime;
		String day="";
		String month="";
		String year="";
		StringTokenizer ST;


		for(i=0;i<numAlerts;++i){
			ST=new StringTokenizer(alertList[i][0],"/");
			day=ST.nextToken();
			month=ST.nextToken();
			year=ST.nextToken();

			alertTime=DateFormat.getInstance().parse(month+"/"+day+"/"+year+ " 00:01 AM, IST").getTime();
			if(alertTime<currentTime){
				returnList[j][0] = alertList[i][0];
				returnList[j][1] = alertList[i][1];
				j++;
			}
		}
		if(debug)printArray(returnList, j);
		return(returnList);
	}

	//This method sets new alerts. It basically registers a new alert as set by the user.
	public void setNewAlert(String date, String details)throws Exception{

		alertList[numAlerts][0] = date;
		alertList[numAlerts][1] = details;
		numAlerts++;

		//Update the alert file. Done immediately for fault tolerance.
		DR.updateAlertsFile(alertList);
	}
	
	//This method deletes a particular alert (specified in terms of the array index)
	public void deleteAlert(int index)throws Exception{
		
		int i;
		//Delete the alert from the alert list
		for(i = index; i<numAlerts-1;++i){
			alertList[i][0] = alertList[i+1][0];
			alertList[i][1] = alertList[i+1][1];
		}
		alertList[i][0] = null;
		
		//make this change in the alerts file
		DR.deleteAlert(index);
		
		numAlerts--;
	}
}
