//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//PIMS: Personal Investment Management System
//Class : GUI
//This is the main class for the implementation of graphical user
//interface. It makes the main gui, then takes in the inputs of the
//user and invokes the corresponding routine.
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.plaf.*;
import java.lang.*;
import java.io.*;
import javax.swing.table.*;
import java.util.*;

public class GUI extends JFrame implements ActionListener{

	private dataRepository DR;
	private Alerts AL;
	private SecurityManager SM;
	private CurrentValueSystem CVS;
	private Investment INV;

	private JScrollPane jsp;
	private JScrollPane jsp2;
	private JScrollPane jsp3=null;
	private Label label[][];
	private JTree tree;
	private JButton b1;
	private JButton b2;
	private JButton b3;
	private JButton b4;
	private JTable table;
	private DefaultMutableTreeNode top;
	private String presentClick="";
	private Panel subPanel=null;

	public int editableRow=-1;
	public Container cp=getContentPane();
	public int length;
	public int numOfFalse;
	public String trans[][];
	public String fieldHeads[];
	public String actualFieldHeads[];
	public String actualTrans[][];

	public GUI(dataRepository DR, Alerts AL, SecurityManager SM, CurrentValueSystem CVS, Investment INV)throws Exception{

		this.DR = DR;
		this.AL = AL;
		this.SM = SM;
		this.CVS = CVS;
		this.INV = INV;
	}
	
	public void makeGUI()throws Exception{
	
		setTitle("PIMS: Pesonal Investment Management System");
		setSize(new Dimension(750,500));
		cp.setBackground(Color.white);
		setBackground(Color.white);
		setVisible(true);

		//add the action listener
		addWindowListener(new MyWindowAdapterGUI());
		Container contentPane=cp;
		cp.setVisible(false);
		
		contentPane.setLayout(new BorderLayout());

		buildMenuBar(contentPane);
		buildPortfolioList(contentPane);
		buildTree(contentPane);
		buildLowerMenu(contentPane);
		cp.setVisible(true);
	}

	void treeMouseClicked(MouseEvent me)throws Exception{
	
		TreePath tp = tree.getPathForLocation(me.getX(),me.getY());
		if(tp == null)return;
		presentClick=tp.toString();
		int count=0;

		for(int i=0;i<presentClick.length();i++){
			if(presentClick.charAt(i)==',')
				count++;
		}

		switch(count){
			case 0:
				b1.setText("Add Portfolio");
				b2.setText("Delete Portfolio");
				b3.setText("Rename Portfolio");
				b4.setText("null");
				b1.setVisible(true);
				b2.setVisible(true);
				b3.setVisible(true);
				b4.setVisible(false);
				buildPortfolioList(cp);
				break;
			case 1:
				b1.setText("Add Security");
				b2.setText("Delete Security");
				b3.setText("Rename Security");
				b4.setText("null");
				b1.setVisible(true);
				b2.setVisible(true);
				b3.setVisible(true);
				b4.setVisible(false);
				String pf_name=presentClick.substring(presentClick.indexOf(',')+2,presentClick.length()-1);
				buildSecurityList(cp,pf_name);
				break;
			case 2:
				b1.setText("Add Transaction");
				b2.setText("Delete Transaction");
				b3.setText("Edit Transaction");
				b4.setText("null");
				b1.setVisible(true);
				b2.setVisible(true);
				b3.setVisible(true);
				b4.setVisible(false);
				String pfName=presentClick.substring(presentClick.indexOf(',')+2,presentClick.lastIndexOf(','));
				String secName=presentClick.substring(presentClick.lastIndexOf(',')+2,presentClick.length()-1);
				buildTransactionDisplay(cp,pfName,secName);
				break;

			default:System.out.println("error in num comma");
				break;
		}
	}

	void lowerMenuActionClicked(ActionEvent me)throws Exception{
		String str=me.getActionCommand();

		if(str.equals("Add Portfolio")){
			AskPortfolioName ask=new AskPortfolioName(INV, str,this);
			ask.setVisible(true);
		}
	}
	public void actionPerformed(ActionEvent ae){
	
		String str=ae.getActionCommand();
		try{
		if(str.equals("Add Portfolio")){
			AskPortfolioName ask=new AskPortfolioName(INV, str,this);
			ask.setVisible(true);

		}
		if(str.equals("Delete Portfolio")){
			AskPortfolioName ask=new AskPortfolioName(INV, str,this);
			ask.setVisible(true);
		}
		if(str.equals("Rename Portfolio")){
			AskPortfolioName ask=new AskPortfolioName(INV, str,this);
			ask.setVisible(true);
		}
		if(str.equals("Add Security")){
			String pf_name=presentClick.substring(presentClick.lastIndexOf(',')+2,presentClick.length()-1);
			AskSecurityName ask=new AskSecurityName(INV, CVS, str,this,pf_name);
			ask.setVisible(true);
		}
		if(str.equals("Delete Security")){
			String pf_name=presentClick.substring(presentClick.lastIndexOf(',')+2,presentClick.length()-1);
			AskSecurityName ask=new AskSecurityName(INV, CVS, str,this,pf_name);
			ask.setVisible(true);
		}
		if(str.equals("Rename Security")){
			String pf_name=presentClick.substring(presentClick.lastIndexOf(',')+2,presentClick.length()-1);
			AskSecurityName ask=new AskSecurityName(INV, CVS, str,this,pf_name);
			ask.setVisible(true);
		}
		if(str.equals("Add Transaction")){
			String pf_name=presentClick.substring(presentClick.indexOf(',')+2,presentClick.lastIndexOf(','));
			String sec_name=presentClick.substring(presentClick.lastIndexOf(',')+2,presentClick.length()-1);
			if(INV.ReturnTypeOfSecurity(pf_name, sec_name)){//bank
				AskBankTransactionDetails ask=new AskBankTransactionDetails(INV, pf_name,sec_name,this);
				ask.setSize(400,400);
				ask.setLocation(120,150);
				ask.setVisible(true);
			}
			else{//share
				AskShareTransactionDetails ask=new AskShareTransactionDetails(INV, pf_name,sec_name,this);
				ask.setSize(400,400);
				ask.setLocation(120,150);
				ask.setVisible(true);
			}
		}
		if(str.equals("Delete Transaction")){
			String pf_name=presentClick.substring(presentClick.indexOf(',')+2,presentClick.lastIndexOf(','));
			String sec_name=presentClick.substring(presentClick.lastIndexOf(',')+2,presentClick.length()-1);
			int selectedRow=table.getSelectedRow();
			if(selectedRow==-1){
				new Error("Click on the transaction that you want to delete");
				return;
			}
			try{
				INV.DeleteTransaction(pf_name,sec_name,selectedRow);
				buildTransactionDisplay(cp,pf_name,sec_name);
			}catch(Exception e){
				new Error("Problem in deleting transaction");
				return;
			}
		}
		if(str.equals("Edit Transaction")){
			String pf_name=presentClick.substring(presentClick.indexOf(',')+2,presentClick.lastIndexOf(','));
			String sec_name=presentClick.substring(presentClick.lastIndexOf(',')+2,presentClick.length()-1);
			int selectedRow=table.getSelectedRow();
			if(selectedRow==-1){
				new Error("Click on the transaction that you want to edit");
				return;
			}
			Transaction TransactionList[] = INV.ReturnTransactionList(pf_name, sec_name);
			if(INV.ReturnTypeOfSecurity(pf_name, sec_name)){//bank
				AskBankEditTransactionDetails ask=new AskBankEditTransactionDetails(INV, TransactionList[selectedRow],pf_name,sec_name,this,selectedRow);
				ask.setVisible(true);
			}
			else{//share
				AskShareEditTransactionDetails ask=new AskShareEditTransactionDetails(INV, TransactionList[selectedRow],pf_name,sec_name,this,selectedRow);
				ask.setVisible(true);
			}
		}
		}catch(Exception e){
			new Error("Unknown exception "+e+" occured");
			return;
		}
		if(str.equals("Exit"))
			System.exit(0);
		if(str.equals("Download Prices")){
		   try{
			CVS.UpdateCurrentPrice();
		   }catch(Exception e){
		   	new Error("Problem in updating current price of shares");
			return;
		   }
		}
		if(str.equals("Download Prices using Proxy")){
		   try{
			System.out.println("Download prices using proxy");
			CVS.UpdateCurrentPrice(true);
		   }catch(Exception e){
		   	new Error("Problem in updating current price of shares");
			return;
		   }
		}
		if(str.equals("Set Alerts")){
		   try{	
		   	new SetAlerts(AL);
		   }catch(Exception e){
		   	new Error("Problem in setting new alert");
			return;
		   }
		}
		if(str.equals("Show Alerts")){
		   try{
			new ShowAlerts(AL);
		   }catch(Exception e){
		   	new Error("Problem in showing pending alerts");
			return;
		   }
		}
		if(str.equals("Delete Alerts")){
		   try{
			new DeleteAlert(AL);
		   }catch(Exception e){
		   	new Error("Problem in deleting alert");
			return;
		   }
		}
		if(str.equals("Change Password")){
		   try{
			new ChangePassword(SM);
		   }catch(Exception e){
		   	new Error("Problem in changing password");
			return;
		   }
		}
		if(str.equals("Help")){
		   try{
			new Help();
		   }catch(Exception e){}
		}
		if(str.equals("About")){
		   try{
			new About();
		   }catch(Exception e){}
		}

		if(str.equals("Edit Prices")){
		   try{
		   	new EditPrices(CVS);
		   }catch(Exception e){
		   	new Error("Problem in editing price");
			return;
		   }
		}
		return;
	}

	void buildMenu(JMenuBar menuBar,String menuName,String menuItems[])throws Exception{
		//Build the first menu
		JMenu menu = new JMenu(menuName);
		menu.setBackground(Color.white);
		menu.setMnemonic(KeyEvent.VK_1);
		menu.getAccessibleContext().setAccessibleDescription("The only menu in this program that has menu items");
        	menuBar.add(menu);

		for(int i=0;i<menuItems.length;i++){
			buildMenuItem(menu,menuItems[i]);
		}

	}

	void buildMenuItem(JMenu menu, String menuItemName)throws Exception{
		JMenuItem menuItem = new JMenuItem(menuItemName,KeyEvent.VK_1);
		menuItem.setBackground(Color.white);
		//menuItem.setRolloverEnabled(true);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.ALT_MASK));
		menuItem.getAccessibleContext().setAccessibleDescription("A menu item");
        	menu.add(menuItem);
		menuItem.addActionListener(this);
	}

	public void buildTree(Container contentPane)throws Exception{
	
		int portfolios_length = 0, securities_length = 0;
		Enumeration e=null;
		if(jsp!=null){
			e=tree.getExpandedDescendants(new TreePath(top));
			contentPane.remove(jsp);
		}
		
		SecurityDS securities[];
		String portfolios[] = INV.ReturnPortfolioList();//file.list();
		while(portfolios[portfolios_length] != null)portfolios_length++;
		top=new DefaultMutableTreeNode("Investment");

		DefaultMutableTreeNode pf;
		DefaultMutableTreeNode sc;
		for(int i=0;i<portfolios_length;i++){
			pf=new DefaultMutableTreeNode(portfolios[i]);			
			securities = INV.ReturnSecurityList(portfolios[i]);
			securities_length = 0;
			while(securities[securities_length] != null)securities_length++;
			for(int j=0;j<securities_length;j++){
				sc=new DefaultMutableTreeNode(securities[j].SecurityName);
				pf.add(sc);
			}
			top.add(pf);
		}

		TreePath tp;
		tree=new JTree(top);

		if(jsp!=null){
			if(e!=null){
				while(e.hasMoreElements()){
					tp=(TreePath)e.nextElement();
						tree.expandPath(tp);
				}
			}
		}

		int v=ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
		int h=ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;

		jsp =new JScrollPane(tree,v,h);

		contentPane.add(jsp,BorderLayout.WEST);
		
		tree.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent me){
				try{
				treeMouseClicked(me);
				}catch(Exception f){System.out.println(f);}
			}

		});

		setVisible(true);
	}

	void buildMenuBar(Container contentPane)throws Exception{
		//menu bar
		contentPane.setVisible(false);

		Panel pan = new Panel();
		pan.setLayout(new GridLayout(2,6));
		Button DP = new Button("Download Prices");
		Button DPP = new Button("Download Prices using Proxy");
		Button EX = new Button("Exit");
		Button AL = new Button("Show Alerts");
		Button SAL = new Button("Set Alerts");
		Button DAL = new Button("Delete Alerts");
		Button CHP = new Button("Change Password");
		Button HLP = new Button("Help");
		Button ABT = new Button("About");
		Button EDT = new Button("Edit Prices");

		DP.addActionListener(this);
		DPP.addActionListener(this);
		EX.addActionListener(this);
		AL.addActionListener(this);
		SAL.addActionListener(this);
		DAL.addActionListener(this);
		CHP.addActionListener(this);
		HLP.addActionListener(this);
		ABT.addActionListener(this);
		EDT.addActionListener(this);

		pan.add(AL);
		pan.add(SAL);
		pan.add(DAL);
		pan.add(DP);
		pan.add(DPP);
		pan.add(EDT);
		pan.add(CHP);
		pan.add(HLP);
		pan.add(ABT);
		pan.add(EX);

		contentPane.add(pan, BorderLayout.NORTH);
		contentPane.setVisible(true);
	}


	public void buildTransactionDisplay(Container contentPane,String pfName,String secName)throws Exception{
	
		if(jsp3!=null)
			contentPane.remove(jsp3);
			
		if(subPanel!=null)
			contentPane.remove(subPanel);
			
		contentPane.setVisible(false);
		Label label=new Label("");
		
		Transaction TransactionList[] = INV.ReturnTransactionList(pfName,secName);
		
		if(TransactionList[0] == null){//No transactions to display
			contentPane.setVisible(true);
			return;
		}
		
		if(TransactionList[0].Bank){
			label.setText("This is a bank");
		}
		else{
			label.setText("This is a share");
		}

		Panel panel=new Panel();
		panel.setBackground(Color.white);
		panel.setLayout(new BorderLayout());
		panel.add(label,BorderLayout.SOUTH);

		//Count the number of transactions
		length = 0;
		while(TransactionList[length] != null)length++;
		int fieldLength = TransactionList[0].FieldLength();
		boolean isShare = (!TransactionList[0].Bank);

		trans = new String[length][fieldLength];
		fieldHeads = TransactionList[0].GetFieldNames();
		
		for(int i=0;i<length;i++){
			trans[i][0] = TransactionList[i].Date();
			trans[i][1] = TransactionList[i].Details;
			if(isShare)trans[i][2] = Integer.toString(TransactionList[i].NumShares);
			else trans[i][2] = String.valueOf(TransactionList[i].AmountTransacted);
			if(isShare)trans[i][3] =String.valueOf( TransactionList[i].CostOfShare);
			else trans[i][3] = (TransactionList[i].TransType)?("debit"):("credit");
			if(isShare)trans[i][4] = (TransactionList[i].TransType)?("sell"):("buy");
		}
		
		TableModel dataModel = new AbstractTableModel() {
			public String getColumnName(int col){
				return fieldHeads[col];
			}
			public int getColumnCount(){
				return fieldHeads.length;
			}
			public int getRowCount(){
				return length;
			}
			public Object getValueAt(int row, int col){
				return trans[row][col];//trans[row][col];//actualTrans[row][col];
			}
			public boolean isCellEditable(int row,int col){
				if(row == editableRow)
					return true;
				else
					return false;
			}
		};
		table = new JTable(dataModel);
		table.setBackground(Color.white);
		int v=ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
		int h=ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;

		jsp3=new JScrollPane(table,v,h);
		jsp3.setBackground(Color.white);
		
		contentPane.add(jsp3,BorderLayout.CENTER);

		contentPane.setVisible(true);
	}

	void buildLowerMenu(Container contentPane)throws Exception{
		contentPane.setVisible(false);
		Panel subcontentPane=new Panel();
		subcontentPane.setBackground(Color.white);
		b1=new JButton("Add Portfolio");
		b2=new JButton("Delete Portfolio");
		b3=new JButton("Rename Portfolio");
		b4=new JButton("null");

		subcontentPane.setLayout(new FlowLayout(5));

		b1.addActionListener(this);
		b2.addActionListener(this);
		b3.addActionListener(this);
		b4.addActionListener(this);
		b1.setBackground(Color.white);
		b2.setBackground(Color.white);
		b3.setBackground(Color.white);
		b4.setBackground(Color.white);
		b1.setVisible(true);
		b2.setVisible(true);
		b3.setVisible(true);
		b4.setVisible(false);

		subcontentPane.add(b1);
		subcontentPane.add(b2);
		subcontentPane.add(b3);
		subcontentPane.add(b4);
		contentPane.add(subcontentPane,BorderLayout.SOUTH);
		contentPane.setVisible(true);
	}

	public void buildPortfolioList(Container contentPane)throws Exception{
	
		int pf_length = 0;
		
		if(subPanel!=null)
			contentPane.remove(subPanel);
			
		if(jsp3!=null)
			contentPane.remove(jsp3);
			
		contentPane.setVisible(false);
		
		subPanel=new Panel();
		subPanel.setBackground(Color.white);
		
		String pf[] = INV.ReturnPortfolioList();
		while(pf[pf_length] != null)pf_length++;
		
		subPanel.setLayout(new GridLayout(Math.max(pf_length,10)+2,2));//4 is added for beautification
		
		label=new Label[Math.max(pf_length,10)+2][2];
		label[0][0]=new Label("Portfolios");
		label[0][0].setFont(new Font("Symbol",Font.BOLD,15));
		label[0][1]=new Label("Worth");
		label[0][1].setFont(new Font("Symbol",Font.BOLD,15));
		
		subPanel.add(label[0][0]);
		subPanel.add(label[0][1]);
		
		for(int i=1;i<pf_length+1;i++){
			label[i][0]=new Label(pf[i-1]);
			subPanel.add(label[i][0]);
			double D = INV.ComputeNetWorth(pf[i-1]);//give the current date here
			label[i][1]=new Label(giveWith2PlacePrecision(String.valueOf(D)));
			subPanel.add(label[i][1]);
		}
		for(int i=pf_length+1;i<Math.max(pf_length,10)+1;i++){
			label[i][0]=new Label(" ");
			label[i][1]=new Label(" ");
			subPanel.add(label[i][0]);
			subPanel.add(label[i][1]);
		}
		label[Math.max(pf_length,10)+1][0]=new Label("Net Worth");
		label[Math.max(pf_length,10)+1][0].setFont(new Font("Symbol",Font.BOLD,15));

		double F = INV.ComputeNetWorth();
		label[Math.max(pf_length,10)+1][1]=new Label(giveWith2PlacePrecision(String.valueOf(F)));
		label[Math.max(pf_length,10)+1][1].setFont(new Font("Symbol",Font.BOLD,15));
		subPanel.add(label[Math.max(pf_length,10)+1][0]);
		subPanel.add(label[Math.max(pf_length,10)+1][1]);
		
		contentPane.add(subPanel,BorderLayout.CENTER);
		contentPane.setVisible(true);
	}

	public void buildSecurityList(Container contentPane,String pf_name)throws Exception{
	
		int sec_length=0;
		if(subPanel!=null)
			contentPane.remove(subPanel);
			
		if(jsp3!=null)
			contentPane.remove(jsp3);
			
		contentPane.setVisible(false);
		
		subPanel=new Panel();
		subPanel.setBackground(Color.white);
		
		SecurityDS sec[] = INV.ReturnSecurityList(pf_name);
		while(sec[sec_length] != null)sec_length++;
		
		subPanel.setLayout(new GridLayout(Math.max(sec_length,10)+2,4));
		
		label=new Label[Math.max(sec_length,10)+2][4];
		label[0][0]=new Label("Securities");
		label[0][0].setFont(new Font("Symbol",Font.BOLD,15));
		label[0][1]=new Label("Worth");
		label[0][1].setFont(new Font("Symbol",Font.BOLD,15));
		label[0][2]=new Label("Monthly ROI");
		label[0][2].setFont(new Font("Symbol",Font.BOLD,15));
		label[0][3]=new Label("Company Name");
		label[0][3].setFont(new Font("Symbol",Font.BOLD,15));
		
		subPanel.add(label[0][0]);
		subPanel.add(label[0][1]);
		subPanel.add(label[0][2]);
		subPanel.add(label[0][3]);
		
		for(int i=1; i < sec_length+1; i++){
		
			label[i][0] = new Label(sec[i-1].SecurityName);
			subPanel.add(label[i][0]);
			
			double D = INV.ComputeNetWorth(pf_name,sec[i-1].SecurityName);
			label[i][1]=new Label(giveWith2PlacePrecision(String.valueOf(D)));
			subPanel.add(label[i][1]);
			double G = INV.ComputeROI(pf_name, sec[i-1].SecurityName);
			label[i][2]=new Label(giveWith2PlacePrecision(String.valueOf(G))+"%");
			subPanel.add(label[i][2]);
			label[i][3] = new Label(sec[i-1].CompanyName);
			subPanel.add(label[i][3]);
		}
		for(int i=sec_length+1;i<Math.max(sec_length,10)+1;i++){
			label[i][0]=new Label(" ");
			label[i][1]=new Label(" ");
			label[i][2]=new Label(" ");
			label[i][3]=new Label(" ");
			subPanel.add(label[i][0]);
			subPanel.add(label[i][1]);
			subPanel.add(label[i][2]);
			subPanel.add(label[i][3]);
		}
		label[Math.max(sec_length,10)+1][0]=new Label("Net Worth");
		label[Math.max(sec_length,10)+1][0].setFont(new Font("Symbol",Font.BOLD,15));
		double F = INV.ComputeNetWorth(pf_name);
		label[Math.max(sec_length,10)+1][1]=new Label(giveWith2PlacePrecision(String.valueOf(F)));
		label[Math.max(sec_length,10)+1][1].setFont(new Font("Symbol",Font.BOLD,15));
		
		subPanel.add(label[Math.max(sec_length,10)+1][0]);
		subPanel.add(label[Math.max(sec_length,10)+1][1]);
		subPanel.add(new Label(""));
		
		contentPane.add(subPanel,BorderLayout.CENTER);
		subPanel.setVisible(true);
		contentPane.setVisible(true);

	}

	String giveWith2PlacePrecision(String number)throws Exception{
		String reply="";
		if(number.indexOf('.')==-1){
			reply=number;
		}
		else{
			reply= number.substring(0,number.indexOf('.')+Math.min(3,number.length()-number.indexOf('.')));
		}

		return reply;
	}
}

class MyWindowAdapterGUI extends WindowAdapter{

	//exiting safely
	public void windowClosing(WindowEvent we){
		System.exit(0);
	}
}
